﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using BuzzGUI.Interfaces;
using System.Collections.Generic;
using Newtonsoft.Json;
using System.Linq;
using System;

namespace Parambulator
{

    public class MachineModel
    {
        public MachineModel(IMachine im)
        {
            machine = im;
        }

        public string name { get; set; }

        [JsonIgnore]
        IMachine machine_;
        [JsonIgnore]
        public IMachine machine
        {
            get { return machine_; }
            set
            {
                if (value == null) return;
                machine_ = value;
                name = machine_.Name;
            }
        }

        public List<ParameterModel> targeted_parameter_models_;
        [JsonIgnore]
        public List<ParameterModel> targeted_parameter_models
        {
            get { return targeted_parameter_models_ == null ? new List<ParameterModel>() : targeted_parameter_models_; }
            set { targeted_parameter_models_ = value; }
        }


        public override string ToString()
        {
            return machine == null ? "" : machine.Name;
        }
    }

    public partial class MachineDisplay : UserControl
    {
        public Action AdjustSizeAction;
        public Action AdjustSizeActionHandler;
        public MachineModel model;

        public MachineDisplay()
        {
            InitializeComponent();
            Loaded += MachineDisplay_Loaded;
        }

        public MachineDisplay(MachineModel mm, Action handler) : this()
        {
            model = mm;
            AdjustSizeActionHandler = handler;
            AdjustSizeAction += handler;
            set_name();
            model.machine.PropertyChanged += Machine_PropertyChanged;
            update_parameters(model.targeted_parameter_models);
        }

        public void prepare_for_deletion()
        {
            AdjustSizeAction -= AdjustSizeActionHandler;
            model.machine.PropertyChanged -= Machine_PropertyChanged;
        }

        private void Machine_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (!(e.PropertyName.Equals("Name"))) return;
            set_name();
        }

        bool loaded_ = false;

        private void MachineDisplay_Loaded(object sender, RoutedEventArgs e)
        {
            if (loaded_) return;
            loaded_ = true;
            parameter_selector_dialog_window = new PersistentWindow();
            parameter_selector_dialog_window.Top = 400;
            parameter_selector_dialog_window.Left = 500;
            parameter_selector_dialog_window.Height = 350;
            parameter_selector_dialog_window.Width = 300;
        }

        public void set_name()
        {
            if (model == null) return;
            machine_name_display.Text = model.ToString();
        }


        private void Triangle_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (triangle_rotation.Angle > 0)
            {
                param_panel.Visibility = Visibility.Collapsed;
                triangle_rotation.Angle = 0;
            }
            else
            {
                param_panel.Visibility = Visibility.Visible;
                triangle_rotation.Angle = 90;
            }
            AdjustSizeAction();
        }

        private void edit_MouseDown(object sender, MouseButtonEventArgs e)
        {
            ParamSelectorDialog psd = new ParamSelectorDialog(model, parameter_selector_dialog_window, update_parameters);
            parameter_selector_dialog_window.Content = psd;
            parameter_selector_dialog_window.Show();
        }

        void update_parameters(List<ParameterModel> new_list)
        {
            model.targeted_parameter_models = new_list;
            foreach (ParameterDisplay pmd in param_panel.Children.Cast<ParameterDisplay>().ToList())
            {
                pmd.prepare_for_deletion();
            }
            param_panel.Children.Clear();
            foreach (ParameterModel pm in new_list)
            {
                param_panel.Children.Add(new ParameterDisplay(pm));
            }
            AdjustSizeAction?.Invoke();
            Statics.UpdateAction();
        }

        PersistentWindow parameter_selector_dialog_window;
    }
}
