﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using BuzzGUI.Common;
using BuzzGUI.Interfaces;

namespace Parambulator
{
 
    public partial class ParamSelectorDialog : UserControl
    {
        public Action<List<ParameterModel>> OKAction;

        MachineModel machine_model;
        IMachine machine { get { return machine_model.machine; } }
        Window my_window;

        public ParamSelectorDialog(MachineModel mm, PersistentWindow w, Action<List<ParameterModel>> handler)
        {
            InitializeComponent();
            machine_model = mm;
            machine_name_display.Text = machine_model.name;
            OKAction += handler;
            w.Closing += (o, e) => OKAction -= handler;
            my_window = w;
            build_selector_display();
        }

        void build_selector_display()
        {
            List<IParameter> current_param_targets = machine_model.targeted_parameter_models.Select(tpm => tpm.parameter).ToList();
            List<IParameter> global_params =  machine.ParameterGroups.Where(pg => pg.Type == ParameterGroupType.Global).FirstOrDefault().Parameters.ToList();
            List<IParameter> track_params = machine.ParameterGroups.Where(pg => pg.Type == ParameterGroupType.Track).FirstOrDefault().Parameters.ToList();
            List<IParameter> all_params = global_params.Union(track_params).ToList();
            foreach (IParameter ip in all_params)
            {
                int trk = 0;
                if (current_param_targets.Contains(ip)) trk = machine_model.targeted_parameter_models.Where(tm => tm.parameter == ip).Select(tm => tm.track).FirstOrDefault();
                param_stack.Children.Add(new ParamSelectorProxy(ip, trk, current_param_targets.Contains(ip)));
            }
        }


        private void OK_Click(object sender, RoutedEventArgs e)
        {
            List<ParamSelectorProxy> proxies = param_stack.Children.Cast<ParamSelectorProxy>().ToList();
            OKAction?.Invoke(proxies.Where(p=>p.is_selected).Select(p=>p.param_model).ToList());
            my_window.Content = null;
            my_window.Close();
        }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            my_window.Content = null;
            my_window.Close();
        }
    }
}
