﻿using Buzz.MachineInterface;
using BuzzGUI.Common;
using BuzzGUI.Interfaces;
using System.Collections.Generic;
using Newtonsoft.Json;
using System.ComponentModel;
using System;
using System.Linq;
using System.Reflection;

namespace Parambulator
{

    [MachineDecl(Name = "Snowglobe Parambulator", ShortName = "Parambe", Author = "snowglobe")]
    public class ParambulatorMachine : IBuzzMachine
    {

        public ParambulatorMachine(IBuzzMachineHost host)
        {
            _host = host;
        }

        
        [ParameterDecl]
        public bool Dummy { get; set; }
        
        List<MachineModel> targets_ = new List<MachineModel>();
        public List<MachineModel> targets
        {
            get { return targets_; }
            set
            {
                targets_ = value;
                update_state();
            }
        }

        State state_ =  new State();
        public State MachineState
        {
            get { return state_ == null ? new State() : state_; }
            set
            {
                state_ = value;
                if (value.serialized_targets == null || value.serialized_targets == "") return;
                try
                {
                    targets = (List<MachineModel>)Statics._unserialize(value.serialized_targets);
                }
                catch (Exception e) { }
            }
        }


        public void ImportFinished(IDictionary<string, string> machineNameMap)
        {
            Global.Buzz.Song.MachineRemoved += Song_MachineRemoved;
            foreach (MachineModel mm in targets)
            {
                mm.machine = Global.Buzz.Song.Machines.Where(mac => mac.Name == mm.name).FirstOrDefault();
                mm.machine.PropertyChanged += Machine_PropertyChanged;
                if (mm.machine == null || mm.targeted_parameter_models == null || mm.targeted_parameter_models.Count() == 0) continue;
                List<IParameter> parameters = new List<IParameter>();
                foreach (IParameterGroup ipg in mm.machine.ParameterGroups)
                {
                    if (ipg.Type == ParameterGroupType.Input) continue;
                    foreach (IParameter ip in ipg.Parameters) parameters.Add(ip); 
                }
                List<string> names = parameters.Select(p => p.Name).ToList();
                foreach (ParameterModel pm in mm.targeted_parameter_models)
                {
                    pm.parameter = parameters.Where(p => p.Name == pm.param_name).FirstOrDefault();
                }
            }
        }

        private void Machine_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (!(e.PropertyName.Equals("Name"))) return;
            IMachine mac = sender as IMachine;
            MachineModel model = targets.Where(t => t.machine == mac).FirstOrDefault();
            if (model == null) return;
            model.name = mac.Name;
            targets = targets.ToList();             //for serialization into State
        }

        private void Song_MachineRemoved(IMachine im)       //if machine is/was target, remove delegates and remove model 
        {
            if (im == _host.Machine)
            {
                Global.Buzz.Song.MachineRemoved -= Song_MachineRemoved;
                return;
            }
            im.PropertyChanged -= Machine_PropertyChanged;
            if (!targets.Select(t => t.machine).Contains(im)) return;
            List<MachineModel> new_targets = targets.Where(m => m.machine != im).ToList();
            targets = new_targets;
        }

        public void update_state()
        {
            MachineState.serialized_targets = Statics._serialize(targets_);
        }

        IBuzzMachineHost _host;
    }

    public static class Statics
    {
        public static Action UpdateAction;

        static JsonSerializerSettings _json_settings = new JsonSerializerSettings { TypeNameHandling = TypeNameHandling.All, ObjectCreationHandling = ObjectCreationHandling.Replace, NullValueHandling = NullValueHandling.Include };

        public static string _serialize(object o)
        {
            return JsonConvert.SerializeObject(o, _json_settings);
        }

        public static object _unserialize(string json)
        {
            return JsonConvert.DeserializeObject<object>(json, _json_settings);
        }

        public static T _clone<T>(T item)
        {
            return (T)_unserialize(_serialize(item));
        }
    }

    public class State
    {
        public string serialized_targets { get; set; }
        public double top { get; set; }
        public double left { get; set; }
        public double height { get; set; }
        public double width { get; set; }

        public State()
        {
            serialized_targets = "";
        }

    }
}
