﻿using BuzzGUI.Common;
using BuzzGUI.Interfaces;
using Buzz.MachineInterface;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Interop;
using System.Collections.ObjectModel;

namespace Parambulator
{
    [MachineGUIFactoryDecl(PreferWindowedGUI = true, IsGUIResizable = true)]
    public class MachineGUIFactory : IMachineGUIFactory
    {
        public IMachineGUI CreateGUI(IMachineGUIHost host) { return new ParambulatorMachineGUI(); }
    }

    public partial class ParambulatorMachineGUI : UserControl, IMachineGUI
    {
        bool loaded_ = false;

        public ParambulatorMachineGUI()
        {
            InitializeComponent();
            Loaded += ParambulatorMachineGUI_Loaded;
        }

        private void ParambulatorMachineGUI_Loaded(object sender, RoutedEventArgs e)
        {
            if (loaded_) return;
            loaded_ = true;
            Statics.UpdateAction += logical_machine.update_state;
            my_window = Window.GetWindow(this);
            my_window.SizeToContent = SizeToContent.WidthAndHeight;
            if (logical_machine.MachineState.top == 0)
            {
                my_window.Top = 300;
                my_window.Left = 400;
                Width = 400;
                adjust_height();
            }
            else
            {
                try
                {
                    my_window.Top = logical_machine.MachineState.top;
                    my_window.Left = logical_machine.MachineState.left;
                    Height = logical_machine.MachineState.height;
                    Width = logical_machine.MachineState.width;
                }
                catch (Exception exc) { }
            }
            my_window.LocationChanged += My_window_LocationChanged;
            my_window.SizeChanged += My_window_SizeChanged;
            machine_selector_dialog_window = new PersistentWindow();
            machine_selector_dialog_window.Top = 400;
            machine_selector_dialog_window.Left = 500;
            machine_selector_dialog_window.Height = 300;
            machine_selector_dialog_window.Width = 200;
            update_targets(logical_machine.targets.Select(m => m.machine).ToList());
            machine_panel.drag_drop_action += rearrangement_detected;
            Global.Buzz.Song.MachineRemoved += Song_MachineRemoved;
        }

        private void My_window_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            logical_machine.MachineState.height = Height;
            logical_machine.MachineState.width = Width;
        }

        private void My_window_LocationChanged(object sender, EventArgs e)
        {
            logical_machine.MachineState.top = my_window.Top;
            logical_machine.MachineState.left = my_window.Left;
        }

        private void Song_MachineRemoved(IMachine mac)
        {
            if (mac == Machine)
            {
                Global.Buzz.Song.MachineRemoved -= Song_MachineRemoved;
                return;
            }
            update_targets(logical_machine.targets.Select(m => m.machine).Where(machine => machine != mac).ToList());
        }

        void rearrangement_detected(FrameworkElement fe)
        {
            logical_machine.targets = machine_panel.items.Cast<MachineDisplay>().ToList().Select(m => m.model).ToList();
        }

        ParambulatorMachine logical_machine;
        IMachine MyMachine_;
        public IMachine Machine
        {
            get { return MyMachine_; }
            set
            {
                if (value == null || value.ManagedMachine == null) return;  //otherwise, errors when closing song
                MyMachine_ = value;
                logical_machine = (ParambulatorMachine)MyMachine_.ManagedMachine;
            }
        }

        private void MachineSelectorClick(object sender, RoutedEventArgs e)
        {
            MachineSelectorDialog mcd = new MachineSelectorDialog(logical_machine.targets.Select(m => m.machine).ToList(), machine_selector_dialog_window, update_targets);
            machine_selector_dialog_window.Content = mcd;
            machine_selector_dialog_window.Show();
        }

        void update_targets(List<IMachine> new_list)
        {
            List<MachineModel> current_models = logical_machine.targets.ToList();
            List<IMachine> keeper_machines = logical_machine.targets.Select(m => m.machine).Where(machine => new_list.Contains(machine)).ToList();
            foreach (MachineModel mm in current_models.ToList())
            {
                if (!keeper_machines.Contains(mm.machine))
                {
                    current_models.Remove(mm);
                    foreach (MachineDisplay md in machine_panel.items.Cast<MachineDisplay>().ToList())
                    {
                        if (md.model.machine == mm.machine)
                        {
                            machine_panel.Remove(md);
                            break;
                        }
                    }
                }
            }
            List<IMachine> current_machines = current_models.Select(m => m.machine).ToList();
            List<IMachine> currently_displayed_machines = machine_panel.items.Cast<MachineDisplay>().ToList().Select(m => m.model.machine).ToList();
            foreach (IMachine im in new_list)
            {
                MachineModel mm = current_models.Where(m => m.machine == im).FirstOrDefault();
                if (mm == null) mm = new MachineModel(im);
                if (!current_machines.Contains(im))
                {
                    current_models.Add(mm);
                }
                if (!currently_displayed_machines.Contains(im))
                {
                    MachineDisplay md = new MachineDisplay(mm, adjust_height);
                    machine_panel.Add(md);
                }
            }
            foreach (MachineDisplay md in machine_panel.items.Cast<MachineDisplay>().Where(md => !new_list.Contains(md.model.machine)))
            {
                md.prepare_for_deletion();
                machine_panel.Remove(md);
            }
            logical_machine.targets = current_models;
            adjust_height();
        }

        public void adjust_height()
        {
            if (my_window == null || !loaded_) return;
            int h = 20 + logical_machine.targets.Count * 22;
            foreach (MachineDisplay md in machine_panel.items.Cast<MachineDisplay>().ToList())
            {
                if (md.param_panel.Visibility == Visibility.Visible) h += 20 * md.model.targeted_parameter_models.Count;
            }
            Height = h;
        }

        PersistentWindow machine_selector_dialog_window;
        Window my_window;

    }
}
