﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using BuzzGUI.Interfaces;
using System.Collections.Generic;
using Newtonsoft.Json;
using System;

namespace Parambulator
{

    public class ParameterModel
    {
        [JsonIgnore]
        public IParameter parameter;

        public ParameterModel(IParameter p)
        {
            parameter = p;
            if (p!= null) param_name = p.Name;
        }

        public string param_name { get; set; }
        public int track { get; set; }

        public override string ToString()
        {
            string track_string = (parameter == null || parameter.Group.Type == ParameterGroupType.Global) ? "g" : track.ToString();
            return parameter == null ? "" : param_name + " [" + track_string + "]";
        }
    }

    public partial class ParameterDisplay : UserControl
    {
        ParameterModel model;

        public IParameter param { get { return model.parameter; } }

        bool loaded_ = false;

        public ParameterDisplay(ParameterModel pm)
        {
            InitializeComponent();
            model = pm;
            param_slider.Minimum = model.parameter.MinValue;
            param_slider.Maximum = model.parameter.MaxValue;
            param_slider.Value = param.GetValue(model.track);
            param_name_display.Text = model.ToString();
            param_name_display.ToolTip = param.Name;
            param_slider.ToolTip = param.Name;
            param_val_display.Text = param.DescribeValue(param_slider.Value);
            Loaded += ParameterDisplay_Loaded;
        }

        private void ParameterDisplay_Loaded(object sender, RoutedEventArgs e)
        {
            if (loaded_) return;
            loaded_ = true;
            try
            {
                param.SubscribeEvents(param.Group.Type == ParameterGroupType.Global ? 0 : model.track, param_val_changed, null);
                param_slider.ValueChanged += param_slider_ValueChanged;
            }
            catch (Exception ex) { }
        }

        void param_val_changed(IParameter ip, int val)
        {
            param_slider.ValueChanged -= param_slider_ValueChanged;
            param_slider.Value = ip.GetValue(val);
            param_val_display.Text = ip.DescribeValue(param_slider.Value);
            param_slider.ValueChanged += param_slider_ValueChanged;
        }

        private void param_slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<int> e)
        {
              param.SetValue(param.Group.Type == ParameterGroupType.Global? 0 : model.track, param_slider.Value);
        }

        private void param_slider_PreviewMouseMove(object sender, MouseEventArgs e)
        {
            DragDropStackPanel.ok_to_drag = false;
        }

        private void param_slider_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            DragDropStackPanel.ok_to_drag = false;
        }

        public void prepare_for_deletion()
        {
            param_slider.ValueChanged -= param_slider_ValueChanged;
            param.UnsubscribeEvents(param.Group.Type == ParameterGroupType.Global ? 0 : model.track, param_val_changed, null);
        }
    }
}
